/**
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
/**
 *
 * @Autores Bruno Teofilo Frassetto
 *          Fernando Augusto Leite
 *
 * -        Instituto Federal de Santa Catarina
 * -        Campus São José
 *
 *          Projeto POO - 2010.1
 *          Jogo de Damas
 * Prof.    Emerson Ribeiro de Mello
 */
package gui;

import java.util.ArrayList;

public class Logica {
    //Atributos da classe

    private Pecas[][] tabuleiro;
    private Boolean jogadorDaVez;
    private ArrayList<String> caminhos;

    public Logica(Pecas[][] tab) {
        //Construtor Padrão
        this.tabuleiro = tab;
        this.jogadorDaVez = true;
        this.caminhos = new ArrayList<String>();

    }

    public int mover(int ox, int oy, int dx, int dy) {
        /*
        Metodo mover serve apenas para testar se o joganor nao está clicando em uma
        casa em branco e com andar() para ver se nao esta andando para trás com uma peca
        normal
         */
        if (((dy % 2 == 0) && (dx % 2 == 1)) || ((dy % 2 == 1) && (dx % 2 == 0))) {
            int res = this.andar(ox, oy, dx, dy);
            //se retornar 0 é possivel a jogada
            return res;
        } else {
            //se retorna 1 não é possivel a jogada
            return 1;
        }
    }

    public void mudarjogador() {
        //somente muda o jogador da vez
        jogadorDaVez = !jogadorDaVez;
    }

    public int andarDama(int ox, int oy, int dx, int dy) {
        /*
        Esse metodo tem uma logica bem simples
        e serve para permitir que a dama so ande na diagonal
        e nao passe por cima de outras pecas
         */

        //direita - cima
        if ((ox > dx) && (oy < dy)) {
            //verifica se está na diagonal
            if ((ox - dx) == (dy - oy)) {
                while ((ox != dx) && (oy != dy)) {
                    if (tabuleiro[dx][dy] == null) {
                        /*
                        Verifica se todas as casas da origem ate o destino sao nulas
                         */
                        dx++;
                        dy--;
                        if ((ox == dx) && (oy == dy)) {
                            return 0;
                        }
                    } else {
                        return 1;
                    }
                }
            }
        }
        //direita - baixo
        if ((ox < dx) && (oy < dy)) {
            //verifica se está na diagonal
            if ((dx - ox) == (dy - oy)) {
                while ((ox != dx) && (oy != dy)) {
                    if (tabuleiro[dx][dy] == null) {
                        /*
                        Verifica se todas as casas da origem ate o destino sao nulas
                         */
                        dx--;
                        dy--;
                        if ((ox == dx) && (oy == dy)) {
                            return 0;
                        }
                    } else {
                        return 1;
                    }
                }
            }
        }
        //esquerda - cima
        if ((ox > dx) && (oy > dy)) {
            //verifica se está na diagonal
            if ((ox - dx) == (oy - dy)) {
                while ((ox != dx) && (oy != dy)) {
                    if (tabuleiro[dx][dy] == null) {
                        /*
                        verifica se todas as casas da origem ate o destino sao nullas
                         */
                        dx++;
                        dy++;
                        if ((ox == dx) && (oy == dy)) {
                            return 0;
                        }
                    } else {
                        return 1;
                    }
                }
            }
        }
        //esquerda - baixo
        if ((ox < dx) && (oy > dy)) {
            //verifica se está na diagonal
            if ((dx - ox) == (oy - dy)) {
                while ((ox != dx) && (oy != dy)) {
                    if (tabuleiro[dx][dy] == null) {
                        /*
                        verifica se todas as casas da origem ate o destino sao nulas
                         */
                        dx--;
                        dy++;
                        if ((ox == dx) && (oy == dy)) {
                            return 0;
                        }
                    } else {
                        return 1;
                    }
                }
            }
        }
        //Se for possível a jogada retorna 0
        //Se não for possível , retorna 1
        return 1;
    }

    public int andar(int ox, int oy, int dx, int dy) {
        /*
        Esse metodo tem a finalidade de nao deixar pecas normais
        andarem para trás e nem andar mais de uma casa para frente
         */

        if (tabuleiro[ox][oy].isRainha() == true) {
            /*
            Se a peca for rainha, é chamado outro metodo
             */
            int retorno = this.andarDama(ox, oy, dx, dy);
            return retorno;
        }
        if (tabuleiro[ox][oy].isCor() == true) {
            if ((dx == ox - 1) && ((dy == oy + 1) || dy == oy - 1)) {
                return 0;
            }
        } else {
            if ((dx == ox + 1) && ((dy == oy + 1) || dy == oy - 1)) {
                return 0;
            }
        }
        //Se for possivel a jogada retorna 0
        //Se não for possivel , retorna 1
        return 1;
    }

    public void setJogadorDaVez(Boolean jogadorDaVez) {
        //Esse metodo serve apenas para setar o jogador da vez
        this.jogadorDaVez = jogadorDaVez;
    }

    public ArrayList<String> procurarTodosCaminhos() {
        /*
        Esse metodo tem a finalidade pegar peca por peca
        e verificar se ela pode ou nao pode comer
        chamando outro metodo - percorreCaminho
         */

        //varre todas as linhas e colunas
        for (int i = 0; i < 8; i++) {
            for (int j = 0; j < 8; j++) {
                //verifica se realmente existe uma peca na casa
                if (tabuleiro[i][j] != null) {
                    //verifica se essa peca é uma peca do jogadorDaVez
                    if (tabuleiro[i][j].isCor() == jogadorDaVez) {
                        //se a peca for uma rainha chama o metodo percorreCaminhoDama
                        if (tabuleiro[i][j].isRainha() == true) {
                            this.percorreCaminhoDama(i, j, i, j, i + "" + j);
                        } else {
                            //se a peca for uma rainha chama o metodo percorreCaminho
                            this.percorreCaminho(i, j, i, j, i + "" + j);
                        }
                    }
                }
            }
        }
        // arraylist com todos os caminhos encontrados
        ArrayList<String> copia = caminhos;

        // zerando caminhos
        caminhos = new ArrayList<String>();

        return copia;

    }

    public void percorreCaminho(int antX, int antY, int xAtual, int yAtual, String anterior) {

        /* verificando direita acima */
        // garantindo que não ocorrerá estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual - 2 >= 0) && (yAtual + 2 <= 7)) && ((xAtual - 2 != antX) || (yAtual + 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual - 1][yAtual + 1] != null) && (tabuleiro[xAtual - 2][yAtual + 2] == null)) {
                if ((tabuleiro[xAtual - 1][yAtual + 1].isCor() != jogadorDaVez)) {
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual + 2))) {
                        // adicionando o caminho percorrido no arraylist

                        String novo = anterior + ":" + (xAtual - 2) + "" + (yAtual + 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual - 2, yAtual + 2, novo);
                    } else {
                        // mensagem de depuração
                        // System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual + 2));
                        //  System.err.println("caminho: " + anterior);
                    }
                }
            }
        }

        /* verificando direita abaixo */
        // garantindo que não ocorrerá estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual + 2 <= 7) && (yAtual + 2 <= 7)) && ((xAtual + 2 != antX) || (yAtual + 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual + 1][yAtual + 1] != null) && (tabuleiro[xAtual + 2][yAtual + 2] == null)) {
                if ((tabuleiro[xAtual + 1][yAtual + 1].isCor() != jogadorDaVez)) {
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual + 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual + 2) + "" + (yAtual + 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual + 2, yAtual + 2, novo);
                    } else {
                        // mensagem de depuração
                        // System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual + 2));
                        // System.err.println("caminho: " + anterior);
                    }
                }
            }
        }


        // esquerda acima
        // garantindo que não ocorrerá estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual - 2 >= 0) && (yAtual - 2 >= 0)) && ((xAtual - 2 != antX) || (yAtual - 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual - 1][yAtual - 1] != null) && (tabuleiro[xAtual - 2][yAtual - 2] == null)) {
                if ((tabuleiro[xAtual - 1][yAtual - 1].isCor() != jogadorDaVez)) {
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual - 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual - 2) + "" + (yAtual - 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual - 2, yAtual - 2, novo);
                    } else {
                        //mensagem de depuração
                        //  System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual - 2));
                        // System.err.println("caminho: " + anterior);
                    }

                }
            }
        }



        // esquerda abaixo
        // garantindo que não ocorrerá estouro dos limites da matriz e que não volte pela casa de onde veio
        if (((xAtual + 2 <= 7) && (yAtual - 2 >= 0)) && ((xAtual + 2 != antX) || (yAtual - 2 != antY))) {
            // verificando se é possível comer
            if ((tabuleiro[xAtual + 1][yAtual - 1] != null) && (tabuleiro[xAtual + 2][yAtual - 2] == null)) {
                if ((tabuleiro[xAtual + 1][yAtual - 1].isCor() != jogadorDaVez)) {
                    // verificando se esta jogada não está presente no caminho percorrido até aqui
                    // proteção contra LOOP
                    if (!anterior.contains(xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual - 2))) {
                        // adicionando o caminho percorrido no arraylist
                        String novo = anterior + ":" + (xAtual + 2) + "" + (yAtual - 2);
                        caminhos.add(novo);
                        // será que deste ponto é possível capturar outra peça? vamos ver
                        this.percorreCaminho(xAtual, yAtual, xAtual + 2, yAtual - 2, novo);
                    } else {
                        //mensagem de depuração
                        //System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual - 2));
                        // System.err.println("caminho: " + anterior);
                    }
                }
            }
        }
    }

    public void percorreCaminhoDama(int antX, int antY, int xAtual, int yAtual, String anterior) {
        /*
        Atributos de seguranca, para impedir o tipo de jogada onde a peca
        6A nao faca a jogada: 6A:3D:1F Matando as Pecas 4C:2E
         */

        /*
        1 - | | | | | | |
        2 - | | | | |x| |
        3 - | | | | | | |
        4 - | | |x| | | |
        5 - | |x| | | | |
        6 - |o| | | | | |
             A B C D E F
         */

        boolean dc = true;
        boolean ec = true;
        boolean eb = true;
        boolean db = true;
        int xAtualReal = xAtual;
        int yAtualReal = yAtual;

        //for para ir varrendo todas as diagonais possíveis
        for (int i = 0; i < 8; i++) {
            /* verificando direita acima */
            // garantindo que não ocorrerá estouro dos limites da matriz e que não volte pela casa de onde veio
            //segurança
            if (dc == true) {
                if (((xAtual - (2 + i) >= 0) && (yAtual + (2 + i) <= 7)) && ((xAtual - (2 + i) != antX) || (yAtual + (2 + i) != antY))) {
                    // verificando se é possível comer
                    //cuidado
                    if ((tabuleiro[xAtual - (1 + i)][yAtual + (1 + i)] != null) && (tabuleiro[xAtual - (2 + i)][yAtual + (2 + i)] != null)) {
                        dc = false;
                    }
                    if ((tabuleiro[xAtual - (1 + i)][yAtual + (1 + i)] != null) && (tabuleiro[xAtual - (2 + i)][yAtual + (2 + i)] == null)) {
                        if ((tabuleiro[xAtual - (1 + i)][yAtual + (1 + i)].isCor() != jogadorDaVez)) {
                            if (((xAtual - i) == xAtualReal && (yAtual + i) == yAtualReal) || (tabuleiro[(xAtual - i)][yAtual + i] == null)) {
                                // verificando se esta jogada não está presente no caminho percorrido até aqui
                                // proteção contra LOOP

                                if (!anterior.contains((xAtual) + "" + (yAtual) + ":" + (xAtual - (2 + i)) + "" + (yAtual + (2 + i)))) {
                                    // adicionando o caminho percorrido no arraylist
                                    String novo = anterior + ":" + (xAtual - (2 + i)) + "" + (yAtual + (2 + i));
                                    caminhos.add(novo);
                                    // será que deste ponto é possível capturar outra peça? vamos ver

                                    this.percorreCaminhoDama((xAtual - i), (yAtual + i), xAtual - (2 + i), yAtual + (2 + i), novo);

                                } else {
                                    // mensagem de depuração
                                    //   System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual + 2));
                                    //   System.err.println("caminho: " + anterior);
                                }
                            }
                        }
                    }
                }
            }
            /* verificando direita abaixo */
            // garantindo que não ocorrerá estouro dos limites da matriz e que não volte pela casa de onde veio
            //segurança
            if (db == true) {
                if (((xAtual + (2 + i) <= 7) && (yAtual + (2 + i) <= 7)) && ((xAtual + (2 + i) != antX) || (yAtual + (2 + i) != antY))) {
                    // verificando se é possível comer
                    if ((tabuleiro[xAtual + (1 + i)][yAtual + (1 + i)] != null) && (tabuleiro[xAtual + (2 + i)][yAtual + (2 + i)] != null)) {
                        db = false;
                    }
                    if ((tabuleiro[xAtual + (1 + i)][yAtual + (1 + i)] != null) && (tabuleiro[xAtual + (2 + i)][yAtual + (2 + i)] == null)) {
                        if ((tabuleiro[xAtual + (1 + i)][yAtual + (1 + i)].isCor() != jogadorDaVez)) {
                            if (((xAtual + i) == xAtualReal && (yAtual + i) == yAtualReal) || (tabuleiro[xAtual + i][yAtual + i] == null)) {

                                // verificando se esta jogada não está presente no caminho percorrido até aqui
                                // proteção contra LOOP
                                if (!anterior.contains((xAtual) + "" + (yAtual) + ":" + (xAtual + (2 + i)) + "" + (yAtual + (2 + i)))) {
                                    // adicionando o caminho percorrido no arraylist
                                    String novo = anterior + ":" + (xAtual + (2 + i)) + "" + (yAtual + (2 + i));
                                    caminhos.add(novo);

                                    // será que deste ponto é possível capturar outra peça? vamos ver
                                    this.percorreCaminhoDama((xAtual + i), (yAtual + i), xAtual + (2 + i), yAtual + (2 + i), novo);


                                } else {
                                    // mensagem de depuração
                                    //  System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual + 2));
                                    //  System.err.println("caminho: " + anterior);
                                }
                            }
                        }
                    }
                }
            }


            // esquerda acima
            // garantindo que não ocorrerá estouro dos limites da matriz e que não volte pela casa de onde veio
            //segurança
            if (ec == true) {
                if (((xAtual - (2 + i) >= 0) && (yAtual - (2 + i) >= 0)) && ((xAtual - (2 + i) != antX) || (yAtual - (2 + i) != antY))) {
                    // verificando se é possível comer
                    if ((tabuleiro[xAtual - (1 + i)][yAtual - (1 + i)] != null) && (tabuleiro[xAtual - (2 + i)][yAtual - (2 + i)] != null)) {
                        ec = false;
                    }
                    if ((tabuleiro[xAtual - (1 + i)][yAtual - (1 + i)] != null) && (tabuleiro[xAtual - (2 + i)][yAtual - (2 + i)] == null)) {
                        if ((tabuleiro[xAtual - (1 + i)][yAtual - (1 + i)].isCor() != jogadorDaVez)) {
                            if (((xAtual - i) == xAtualReal && (yAtual - i) == yAtualReal) || (tabuleiro[xAtual - i][yAtual - i] == null)) {
                                // verificando se esta jogada não está presente no caminho percorrido até aqui
                                // proteção contra LOOP
                                if (!anterior.contains((xAtual) + "" + (yAtual) + ":" + (xAtual - (2 + i)) + "" + (yAtual - (2 + i)))) {
                                    // adicionando o caminho percorrido no arraylist
                                    String novo = anterior + ":" + (xAtual - (2 + i)) + "" + (yAtual - (2 + i));
                                    caminhos.add(novo);

                                    // será que deste ponto é possível capturar outra peça? vamos ver
                                    this.percorreCaminhoDama((xAtual - i), (yAtual - i), xAtual - (2 + i), yAtual - (2 + i), novo);
                                } else {
                                    //mensagem de depuração
                                    //  System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual - 2) + "" + (yAtual - 2));
                                    // System.err.println("caminho: " + anterior);
                                }
                            }
                        }
                    }
                }
            }



            // esquerda abaixo
            // garantindo que não ocorrerá estouro dos limites da matriz e que não volte pela casa de onde veio
            //segurança
            if (eb == true) {
                if (((xAtual + (2 + i) <= 7) && (yAtual - (2 + i) >= 0)) && ((xAtual + (2 + i) != antX) || (yAtual - (2 + i) != antY))) {
                    // verificando se é possível comer
                    if ((tabuleiro[xAtual + (1 + i)][yAtual - (1 + i)] != null) && (tabuleiro[xAtual + (2 + i)][yAtual - (2 + i)] == null)) {
                        eb = false;
                    }
                    if ((tabuleiro[xAtual + (1 + i)][yAtual - (1 + i)] != null) && (tabuleiro[xAtual + (2 + i)][yAtual - (2 + i)] == null)) {

                        if ((tabuleiro[xAtual + (1 + i)][yAtual - (1 + i)].isCor() != jogadorDaVez)) {
                            if (((xAtual + i) == xAtualReal && (yAtual - i) == yAtualReal) || (tabuleiro[xAtual + i][yAtual - i] == null)) {
                                // verificando se esta jogada não está presente no caminho percorrido até aqui
                                // proteção contra LOOP
                                if (!anterior.contains((xAtual) + "" + (yAtual) + ":" + (xAtual + (2 + i)) + "" + (yAtual - (2 + i)))) {
                                    // adicionando o caminho percorrido no arraylist
                                    String novo = anterior + ":" + (xAtual + (2 + i)) + "" + (yAtual - (2 + i));
                                    caminhos.add(novo);
                                    // será que deste ponto é possível capturar outra peça? vamos ver
                                    this.percorreCaminhoDama((xAtual + i), (yAtual - i), xAtual + (2 + i), yAtual - (2 + i), novo);
                                } else {
                                    //mensagem de depuração
                                    // System.err.println("teste  : " + xAtual + "" + yAtual + ":" + (xAtual + 2) + "" + (yAtual - 2));
                                    //  System.err.println("caminho: " + anterior);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
