/**
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
/**
 *
 * @Autores Bruno Teofilo Frassetto
 *          Fernando Augusto Leite
 *
 * -        Instituto Federal de Santa Catarina
 * -        Campus São José
 *
 *          Projeto POO - 2010.1
 *          Jogo de Damas
 * Prof.    Emerson Ribeiro de Mello
 */
package gui;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

public class Painel extends JPanel {

    //atributos da classe
    private ImageIcon vermelha;// - peca vermelha
    private ImageIcon amarela;//- peca azul
    private ImageIcon azulSelecionada;//- peca azul selecionada
    private ImageIcon vermelhoSelecionado;//- peca vermelha selecionada
    private ImageIcon atacaAzul;//- peca azul avisando que tem que atacar
    private ImageIcon atacaVermelha;//- peca vermelha avisando que tem que atacar
    private ImageIcon damaVermelha;//- Dama vermelha
    private ImageIcon damaAzul;//- Dama azul
    private ImageIcon alvo;//destino das pecas
    private boolean selecionado;
    private boolean preenchido;
    private Pecas[][] tabuleiro;
    private int xanterior;
    private int yanterior;
    private Logica logicaJogo;
    private boolean jogadorDaVez;
    private Principal pai;

    public Painel(Principal p) {
        //Construtor padrao
        vermelha = new ImageIcon(this.getClass().getResource("/resources/face-angry.png"));
        amarela = new ImageIcon(this.getClass().getResource("/resources/face-worried.png"));
        vermelhoSelecionado = new ImageIcon(this.getClass().getResource("/resources/face-angry2.png"));
        azulSelecionada = new ImageIcon(this.getClass().getResource("/resources/face-worried2.png"));
        atacaVermelha = new ImageIcon(this.getClass().getResource("/resources/face-angry3.png"));
        atacaAzul = new ImageIcon(this.getClass().getResource("/resources/face-worried3.png"));
        damaVermelha = new ImageIcon(this.getClass().getResource("/resources/dama-angry.png"));
        damaAzul = new ImageIcon(this.getClass().getResource("/resources/dama-worried.png"));
        alvo = new ImageIcon(this.getClass().getResource("/resources/alvo.png"));
        // alterar a cor de fundo do painel
        this.setBackground(Color.WHITE);
        //determinando preenchido e selecionado com false
        //preenchido para desenhar tabuleiro
        //selecionado, para o jogo começar sem nenhuma peca selecionada
        this.preenchido = false;
        this.selecionado = false;
        this.pai = p;
        //criando vetor de pecas
        this.tabuleiro = new Pecas[8][8];
        //chamando metodo para criar as pecas;
        this.criarPecas();
        //determinando jogador da vez como true - azul
        this.jogadorDaVez = true;
        //criando objeto logicaJogo
        this.logicaJogo = new Logica(tabuleiro);
        //iniciando variáveis
        this.xanterior = -1;
        this.yanterior = -1;
    }

    @Override
    public void paint(Graphics g) {
        //iniciando o Graphics g
        super.paint(g);
    }

    public void limpar() {
        //limpa o tabuleiro todo
        this.repaint();
    }

    public boolean isJogadorDaVez() {
        //metodo criado so para se obter qual é o jogador da vez
        return jogadorDaVez;
    }

    public void desenharTabuleiro() {
        /*
        Esse metodo é ultilizado para desenhar o tabuleiro
        pode ser utilizado quando o programa é iniciado
        ou apenas para limpar a tela,
        remover pecas que ja foram mortas, etc...
         */
        Graphics g = this.getGraphics();

        /*
        O tamanho do tabuleiro é 400 por 400
        e o tamanho de cada quadrado é 50 por 50
         */
        for (int i = 0; i < 400; i += 50) {
            for (int j = 0; j < 400; j += 50) {
                if (this.preenchido == false) {
                    g.drawRect(i, j, 50, 50);
                } else {
                    g.fillRect(i, j, 50, 50);
                }
                this.preenchido = !this.preenchido;
            }
            // é criado um tabuleiro 8 linhas por 8 colunas
            this.preenchido = !this.preenchido;
        }
        //marca selecionado como false
        this.selecionado = false;
    }

    public void selecionar(int y, int x) {
        //Esse metodo so é utilizado quando nao há a existência de pecas para comer.
        //Sua funcao é selecionar e mover pecas

        //iniciando o Graphics g
        Graphics g = this.getGraphics();

        //verificando se existe alguma peca ja selecionada
        if (this.selecionado == true) {
            /*
            Se sim, chama o metodo mover. Sua finalidade e ver se o movimento
            que o jogador fez , é um movimento possível.
            Ele envia as linhas e colunas da peca selecionada e destino selecionado
            se retornar 0 - significa que a jogada é possivel
             */
            if (logicaJogo.mover(xanterior, yanterior, x, y) == 0) {
                //verifica se o destino é uma casa em branco
                if (this.tabuleiro[x][y] == null) {
                    //muda peca de lugar, a origem se torna o destino
                    this.tabuleiro[x][y] = this.tabuleiro[xanterior][yanterior];
                    //apaga a peca origem
                    this.tabuleiro[xanterior][yanterior] = null;
                    //set no Historico a jogada realizada
                    this.pai.Historico(x, y, xanterior, yanterior);
                    //desenha o tabuleiro
                    this.desenharTabuleiro();
                    //muda o jogador da vez
                    this.jogadorDaVez = !this.jogadorDaVez;
                    //muda o jogador tambem na classe logica
                    logicaJogo.mudarjogador();
                    //verifica se com o fim da jogada existe alguma dama
                    verificaDama();
                    //preenche todas as pecas
                    preencherPecas();
                } else {
                    /*
                    Se o destino da jogada não for uma casa em branco
                    muda o selecionado para false
                     */
                    this.selecionado = false;
                    //desenha o tabuleiro e preenche todas as pecas
                    this.desenharTabuleiro();
                    this.preencherPecas();
                }
            } else {
                /*
                Se o logicaJogo.mover retornar 1 - significa que a jogada
                nao pode ser realizada, muda o selecionado para false
                 */
                this.selecionado = false;
                //desenha o tabuleiro e preenche todas as pecas
                this.desenharTabuleiro();
                this.preencherPecas();
            }
            //verifica se com o fim da jogada existe alguma dama
            verificaDama();

        } else {
            /*
            Se o selecionado for igual a false significa que nenhuma peca ainda
            foi selecionada.
            Essa segunda parte do metodo serve apenas para selecionar a peca
            recebendo coordenadas que o jogador clicou e armazenando em x e y
             */
            int lin = x;
            int col = y;
            /*
            verificando se a pessoa clicou em uma casa diferente de null,
            ou seja, uma casa em branco ou sem peca
             */
            if (this.tabuleiro[x][y] != null) {
                //verifica se a peca selecionada é a peca do jogador da vez
                if (tabuleiro[x][y].isCor() == this.jogadorDaVez) {
                    /*
                    As coordenadas das pecas selecionadas agora ficam armazendas
                    em xanterior e yanterior para serem utilizadas quando
                    selecionado for igual a true
                     */
                    xanterior = x;
                    yanterior = y;
                    //seta no programa pricipal a peca selecionada
                    this.pai.setjLabel2(x, y);
                    //muda a imagem da peca selecionada
                    //faz um teste para verificar qual imagem deve ser colocada
                    if (tabuleiro[x][y].isCor() == false) {
                        g.drawImage(vermelhoSelecionado.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
                        //determina selecionado igual a true
                        this.selecionado = true;
                    } else {
                        g.drawImage(azulSelecionada.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
                        //determina selecionado igual a true
                        this.selecionado = true;
                    }
                }
            }
        }
    }

    public void verificaDama() {
        /*
        Varre toda a primeira linha e toda ultima linha para
        verificar a existencia de damas.
        Gostaria de lembrar, que para uma peca virar dama, ela tem que terminar
        a jogada na primeira ou ultima linha. Somente a passagem por essas linhas
        nao lhe dao a prioridade de dama.
        Assim, quando se come mais de uma peca e houver a passagem pelo caminho
        que deveria torna uma peca dama, ela continua sendo uma peca normal
         */

        //compara jogador da vez com true para verificar se testa a primeira ou ultima linha
        if (this.jogadorDaVez == true) {
            //primeira linha - teste para pecas azuis
            int lin = 0;
            //varre todas as colunas
            for (int col = 0; col < 8; col++) {
                //verifica se as pecas sao diferentes de Null e azuis
                if (tabuleiro[lin][col] != null && tabuleiro[lin][col].isCor() == true) {
                    //Se sim -  ela se torna uma dama
                    tabuleiro[lin][col].setRainha(true);
                }
            }
        } else {
            //ultima linha - teste para pecas vermelhas
            int lin = 7;
            //varre todas as colunas
            for (int col = 0; col < 8; col++) {
                //verifica se as pecas sao diferentes de Null e vermelhas
                if (tabuleiro[lin][col] != null && tabuleiro[lin][col].isCor() == false) {
                    //Se sim -  ela se torna uma dama
                    tabuleiro[lin][col].setRainha(true);
                }
            }
        }
        //desenha um novo tabuleiro
        desenharTabuleiro();
        //preenche todas as pecas
        preencherPecas();
    }

    public void obrigacomertabuleiro(int linh, int colu) {
        /*
        Esse metodo só tem a função de avisar ao jogador que peca(s) ele
        tem que mover, com fins de evitar perda de tempo do jogador
         */

        //iniciando o Graphics g
        Graphics g = this.getGraphics();
        //a linha e a coluna da peca que tem que ser selecionada
        int lin = linh;
        int col = colu;
        //mudando o desenho da peca jogo
        if (this.jogadorDaVez == true) {
            //para pecas vermenhas
            g.drawImage(atacaAzul.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
        } else {
            //para pecas azuis
            g.drawImage(atacaVermelha.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
        }
    }

    public String melhorCaminho() {
        /*
        Esse metodo pega o arrayList do metodo naoComeParaTraz
        que é o arrayList PercorrerTodoscaminhos ja filtrado
        e separa os melhores caminhos
         */
        int maior = -1;
        ArrayList<String> ca = naoComeParaTraz();
        StringBuffer maiores = new StringBuffer();
        for (String linha : ca) {
            //compara o tamanho da linha atual com a maior que ja se obteve
            if (linha.length() > maior) {
                //se for maior, que a maior, se apaga o "maiores"
                maiores = new StringBuffer(linha + "\n");
                //se adiciona o maior tamanho de linha na String
                maior = linha.length();
            } else if (linha.length() == maior) {
                /*se o tamanho da linha for do mesmo tamanho da String
                se adiciona uma nova linha Na String*/
                maiores.append(linha + "\n");
            }

        }
        //retorna a String somente com os maiores caminhos e ja filtrados.
        return maiores.toString();
    }

    public String dividindoMelhorCaminho() {
        /*
        Esse metodo tem 2 finalidades.
        A primeira é retornar uma String para a classe principal
        para ser avaliado e testado lá.
        A segunda é separar linha e coluna
        e mandar para o metodo obrigacomertabuleiro
         */
        String dividindoMelhorCaminho = this.melhorCaminho().trim();
        //Dividindo o melhor caminho
        String[] array = dividindoMelhorCaminho.toString().split("\n");

        for (String s : array) {
            String teste = s.toString();
            //evitando a entrada de Strings em branco
            if (!teste.equals("")) {
                //Capturando o primeiro x e o primeiro y e convertendo para inteiros
                String oriLin = teste.trim().substring(0, 1);
                String oriCol = teste.trim().substring(1, 2);
                int Lin = Integer.parseInt(oriLin);
                int Col = Integer.parseInt(oriCol);
                //Chamando metodo obrigacomertabuleiro
                this.obrigacomertabuleiro(Lin, Col);
            }
        }
        return dividindoMelhorCaminho;
    }

    public ArrayList<String> naoComeParaTraz() {
        /*
        A função desse metodo é funcionar como um filtro
        que nao permita que as pecas normais comam a primeira vez para traz
         */

        StringBuffer sa = new StringBuffer();
        /*
        Chama o metodo procurarTodosCaminhos
        que retorna todos os caminhos possíveis
         */
        ArrayList<String> ca = logicaJogo.procurarTodosCaminhos();
        ArrayList<String> naoComeParaTraz = new ArrayList<String>();

        for (String linha : ca) {
            //O for varre todas as Strings do arrayList

            //Separa somente o x da primeira peca
            String oriLin1 = linha.trim().substring(0, 1);
            //Separa somente o x da segunda peca
            String oriLin2 = linha.substring(3, 4);
            //Separa somente o y da primeira peca
            String oriCol1 = linha.trim().substring(1, 2);
            //converte todos para inteiros
            int Lin1 = Integer.parseInt(oriLin1);
            int Lin2 = Integer.parseInt(oriLin2);
            int Col1 = Integer.parseInt(oriCol1);

            /*
            Se faz o teste abaixo para previnir
            contra alguns casos onde nao existirá peca nenhuma
            e deveria existir uma rainha.
             */
            if (this.tabuleiro[Lin1][Col1] == null) {
                naoComeParaTraz.add(linha);
            } else {
                //Se for rainha não precisa passar pelo filtro.
                if (this.tabuleiro[Lin1][Col1].isRainha() == true) {
                    naoComeParaTraz.add(linha);
                } else {
                    /*
                    Mas caso contrario, antes de adicionar alguma linha,
                    temos que ter certeza que a peca comecou comendo para frente.
                    E esses são os testes...
                     */
                    if (this.jogadorDaVez == true) {
                        if (Lin1 > Lin2) {
                            //adiciona a linha
                            naoComeParaTraz.add(linha);
                        }
                    } else {
                        if (Lin1 < Lin2) {
                            //adiciona a linha
                            naoComeParaTraz.add(linha);
                        }
                    }
                }
            }
        }
        //retorna arrayList totalmente filtrado
        return naoComeParaTraz;
    }

    public void apagandoPecas(String deletar) {
        //metodo para apagar pecas
        //ve o tamanho da String
        int tamanhoDaString = deletar.length();
        //com esse algoritimo, calculamos quantas pecas vao ser mortas
        int quantPecasDel = (tamanhoDaString - 2) / 3;
        //inicia variavel pecasdel em 0
        int pecasdel = 0;
        //pega o valor da linha da da peca
        String lll1 = deletar.trim().substring(0, 1);
        //pega o valor da linha da da coluna
        String ccc1 = deletar.trim().substring(1, 2);
        //converte os 2 para inteiros
        int ll1 = Integer.parseInt(lll1);
        int cc1 = Integer.parseInt(ccc1);

        while (quantPecasDel != pecasdel / 3) {
            /*
            Enquanto a quantidade de pecas para deletar nao for igual
            a quantidade de pecas deletadas, fica rodando o programa
             */

            // -------------------------------------------------------
            //aqui ele pega sempre o primeiro e o segundo valor ,
            //depois o quarto e o quinto
            //na proxima rodada o quarto e o quinto e depois o setimo e o oitavo
            //e assim por diante

            //exemplo: 12:34:56
            //na primeira rodada ele pega o 1 2 depois o 3 4
            //na segunda rodada o 3 2 e depois o 5 6
            //com essa logica pode se obter as pecas mortas com o simples calculo
            //(linha1+linha3)/2=linhaDaPecaMorta (1+3)/2=2
            //(coluna2+coluna4)/2=colunaDaPecaMorta (2+4)/2=3 => pecaMorta[2][3]
            //segunda rodada
            //(linha4+linha7)/2=linhaDaPecaMorta (3+5)/2=4
            //(coluna5+coluna8)/2=colunaDaPecaMorta (4+6)/2=5 => pecaMorta[4][5]

            // [12 : 34 : 56] - casa que a peca passou
            //   [ 23 : 45] - pecas mortas
            // -------------------------------------------------------

            String pl1 = deletar.trim().substring(pecasdel, pecasdel + 1);
            String pc1 = deletar.trim().substring(pecasdel + 1, pecasdel + 2);
            String pl2 = deletar.trim().substring(pecasdel + 3, pecasdel + 4);
            String pc2 = deletar.trim().substring(pecasdel + 4, pecasdel + 5);
            //muda para inteiro
            int pll1 = Integer.parseInt(pl1);
            int pcc1 = Integer.parseInt(pc1);
            int pll2 = Integer.parseInt(pl2);
            int pcc2 = Integer.parseInt(pc2);
            //incrementa + 3 nos pecas del
            pecasdel = pecasdel + 3;


            // -------------------------------------------------------
            //Porem a logica a cima não funciona para damas
            //Para isso usamos essa parte do programa
            //Tambem é muito simples, ela observa de onde a dama veio
            //e apaga uma peca anterior
            //Ex: a dama acabou de comer na direção direita baixo
            //Ele usa o destino final da dama e apaga uma peca anterior
            //peca[xatual-1][yatual-1]
            //lembrando que isso tambem so funciona porque obrigamos
            //a dama a parar uma casa depois de comer
            // -------------------------------------------------------
            if (this.tabuleiro[ll1][cc1].isRainha() == true) {

                //direita baixo
                if (pll2 > pll1 && pcc2 > pcc1) {
                    //apenas faz um som quando uma peca é morta
                    java.awt.Toolkit.getDefaultToolkit().beep();
                    this.tabuleiro[pll2 - 1][pcc2 - 1] = null;
                }
                //esquerda cima
                if (pll2 < pll1 && pcc2 < pcc1) {
                    //apenas faz um som quando uma peca é morta
                    java.awt.Toolkit.getDefaultToolkit().beep();
                    this.tabuleiro[pll2 + 1][pcc2 + 1] = null;
                }
                //direita cima
                if (pll2 > pll1 && pcc2 < pcc1) {
                    //apenas faz um som quando uma peca é morta
                    java.awt.Toolkit.getDefaultToolkit().beep();
                    this.tabuleiro[pll2 - 1][pcc2 + 1] = null;
                }
                //esquerda baixo
                if (pll2 < pll1 && pcc2 > pcc1) {
                    //apenas faz um som quando uma peca é morta
                    java.awt.Toolkit.getDefaultToolkit().beep();
                    this.tabuleiro[pll2 + 1][pcc2 - 1] = null;
                }


            } else {
                //E se não for dama, faz o calculo normal
                int linh = (pll1 + pll2) / 2;
                int colu = (pcc1 + pcc2) / 2;
                //apenas faz um som quando uma peca é morta
                java.awt.Toolkit.getDefaultToolkit().beep();
                //apaga peca
                this.tabuleiro[linh][colu] = null;
            }
        }

    }

    public void mudandoPecaDeLugar(int xorigem, int yorigem, int xdestino, int ydestino) {
        //metodo com uma única e simples função de mudar a peca de lugar
        //testa para ver se a origem nao é uma casa em branco
        if (this.tabuleiro[xorigem][yorigem] == null) {
        } else {
        }
        //muda peca de lugar
        this.tabuleiro[xdestino][ydestino] = this.tabuleiro[xorigem][yorigem];
        //apaga peca anterior
        this.tabuleiro[xorigem][yorigem] = null;
    }

    public void atacar(int y, int x) {
        /*
        Esse metodo é muito parecido com o selecionar,
        porém so é utilizado quando houver pecas para comer
         */

        //iniciando o Graphics g
        Graphics g = this.getGraphics();

        //verifica se ja existe alguma peca selecionada
        if (this.selecionado == true) {
            /*
            Chama o metodo melhorCaminho para verificar qual(quais)
            é(sao) o(s) melhor(es) caminho(s)
             */

            //.trim serve para tirar a sujeira
            String caminhos = this.melhorCaminho().trim();
            //.split("\n") ser para dividir as Strings por linha
            String[] array = caminhos.toString().split("\n");
            //Varre todas os maiores caminhos
            for (String s : array) {
                String teste = s.toString();

                if (!teste.equals("")) {
                    //Separa linha e coluna origem e destino
                    String oriLin = teste.trim().substring(0, 1);
                    String oriCol = teste.trim().substring(1, 2);// 01
                    String destLin = teste.trim().substring(teste.length() - 2, teste.length() - 1);
                    String destCol = teste.trim().substring(teste.length() - 1);// 23
                    //converte para inteiros
                    int Lin = Integer.parseInt(oriLin);
                    int Col = Integer.parseInt(oriCol);
                    int Lindest = Integer.parseInt(destLin);
                    int Coldest = Integer.parseInt(destCol);
                    /*
                    Testa e ve se a peca selecionada faz parte de alguma String
                    de melhores caminhos
                     */
                    if ((this.xanterior == Lin) && (this.yanterior == Col)) {
                        /*
                        Verifica se o destino faz parte da mesma String de melhores caminhos
                        que a origem
                         */
                        if ((x == Lindest) && (y == Coldest)) {
                            //chama metodo apagando pecas
                            this.apagandoPecas(s);
                            //chama metodo para mudar peca de lugar
                            this.mudandoPecaDeLugar(Lin, Col, Lindest, Coldest);
                            //seta no Historico a jogada
                            this.pai.Historico(x, y, xanterior, yanterior);
                            //muda o jogador da vez
                            this.jogadorDaVez = !this.jogadorDaVez;
                            //muda o jogador da vez no logicaJogo
                            logicaJogo.mudarjogador();
                        }
                    }
                }
            }
            //desenha e preenche pecas no tabuleiro
            this.desenharTabuleiro();
            this.preencherPecas();
            this.dividindoMelhorCaminho();
            //marca selecionado como false
            this.selecionado = false;
            //verica se existe alguma dama
            verificaDama();
        } else {
            /*
            Caso nenhuma peca estiver selecionada ainda,
            pegas as coordenadas do click do mouse
             */
            int lin = x;
            int col = y;

            if (this.tabuleiro[x][y] != null) {
                //verifica se o jogador nao clicou em uma casa branca ou sem peca
                if (tabuleiro[x][y].isCor() == this.jogadorDaVez) {
                    //verifica se a peca é dele

                    //armazena as variaveis de origem
                    xanterior = x;
                    yanterior = y;
                    //seta no programa principal a peca selecionada
                    this.pai.setjLabel2(x, y);
                    //muda imagem da peca selecionada
                    if (tabuleiro[x][y].isCor() == true) {
                        g.drawImage(azulSelecionada.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
                        //marca selecionado igual a true
                        this.selecionado = true;
                    } else {
                        g.drawImage(vermelhoSelecionado.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
                        //marca selecionado igual a true
                        this.selecionado = true;
                    }
                    /*
                    Chama o metodo melhorCaminho para verifica qual(quais)
                    é(sao) o(s) melhor(es) caminho(s)
                     */

                    //.trim serve para tirar a sujeira
                    String caminhos = this.melhorCaminho().trim();
                    //.split("\n") ser para dividir as Strings por linha
                    String[] array = caminhos.toString().split("\n");
                    //varre todo os maiores caminhos
                    for (String s : array) {
                        String teste = s.toString();
                        if (!teste.equals("")) {
                            //Separa linha e coluna origem e destino
                            String oriLin = teste.trim().substring(0, 1);
                            String oriCol = teste.trim().substring(1, 2);// 01
                            String destLin = teste.trim().substring(teste.length() - 2, teste.length() - 1);
                            String destCol = teste.trim().substring(teste.length() - 1);// 23
                            //converte para inteiros
                            int Lin = Integer.parseInt(oriLin);
                            int Col = Integer.parseInt(oriCol);
                            int Lindest = Integer.parseInt(destLin);
                            int Coldest = Integer.parseInt(destCol);
                            /*
                            Testa se a peca selecionada é uma das pecas de
                            maiores caminhos
                             */
                            if ((this.xanterior == Lin) && (this.yanterior == Col)) {
                                //se sim, imprime o destino na tela
                                g.drawImage(alvo.getImage(), 9 + (50 * (Coldest)), 9 + (50 * (Lindest)), this);
                            }
                        }
                    }
                }
            }
        }
    }

    public void criarPecas() {
        //metodo que so tem como função criar pecas a cada novo jogo

        //varre todas as linhas e colunas e apaga todas as pecas
        for (int lin = 0; lin
                < 8; lin++) {
            for (int col = 0; col
                    < 8; col++) {
                this.tabuleiro[lin][col] = null;
            }
        }
        //cria novamente todas as pecas
        for (int lin = 0; lin < 8; lin++) { //percorrendo linhas
            boolean cor = (lin > 4) ? true : false;// determinando a cor da peca
            for (int col = 0; col < 8; col++) { //percorrendo colunas
                if ((lin != 3 && lin != 4) && (((lin % 2 == 0) && (col % 2 == 1)) || ((lin % 2 == 1) && (col % 2 == 0)))) {
                    this.tabuleiro[lin][col] = new Pecas(cor, false);
                }
            }
        }
    }

    public void setJogadorDaVez(boolean jogadorDaVez) {
        //metodo para setar o jogador da vez na classe Painel e Logica
        this.jogadorDaVez = jogadorDaVez;
        logicaJogo.setJogadorDaVez(jogadorDaVez);

    }

    public void fimDeJogo() {
        //Metodo para verificar se algum dos jogares nao tem mais pecas
        Graphics g = this.getGraphics();
        //inicia variaveis
        int pecaAzul = 0;
        int pecaVermelha = 0;
        /*
        Varre todas as linhas e todas as colunas contando a quantidade
        de pecas de cada jogador
         */
        for (int lin = 0; lin < 8; lin++) {
            for (int col = 0; col < 8; col++) {
                if (this.tabuleiro[lin][col] != null) {
                    if (tabuleiro[lin][col].isCor() == true) {
                        pecaAzul++;
                    } else {
                        pecaVermelha++;
                    }
                }
            }
        }
        //determinando uma fonte
        g.setFont(new java.awt.Font("Arial Black", 60, 60));

        //imprindo na tela Fim de Jogo
        if (pecaAzul == 0) {
            g.setColor(Color.red);
            g.drawString("Fim de Jogo", 1, 150);
            g.drawString("Vermelho", 20, 250);
            g.drawString("Ganhou", 30, 350);
        }
        if (pecaVermelha == 0) {
            g.setColor(Color.blue);
            g.drawString("Fim de Jogo", 1, 150);
            g.drawString("Azul", 20, 250);
            g.drawString("Ganhou", 30, 350);


        }
    }

    public void preencherPecas() {
        //metodo que tem como finalidade imprimir cada peca no tabuleiro

        Graphics g = this.getGraphics();
        //varre todas as linhas e todas as colunas
        for (int lin = 0; lin
                < 8; lin++) {
            for (int col = 0; col
                    < 8; col++) {
                //verifica se existe uma peca no lugar
                if (this.tabuleiro[lin][col] != null) {
                    //verifica qual jogador é para cada tipo de imagem
                    if (tabuleiro[lin][col].isCor() == false) {
                        //verifica se é rainha porque a imagem é diferente
                        if (tabuleiro[lin][col].isRainha() == true) {
                            g.drawImage(damaVermelha.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
                        } else {
                            g.drawImage(vermelha.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
                        }
                    } else {
                        //verifica se é rainha porque a imagem é diferente
                        if (tabuleiro[lin][col].isRainha() == true) {
                            g.drawImage(damaAzul.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
                        } else {
                            g.drawImage(amarela.getImage(), 10 + (50 * (col)), 10 + (50 * (lin)), this);
                        }
                    }
                }
            }
        }
    }
}
